'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var KOS = _interopDefault(require('kos-core'));
var $ = _interopDefault(require('lodash'));
var _regeneratorRuntime = _interopDefault(require('babel-runtime/regenerator'));

/**
 * 表单校验数据常量
 */
var XFORM_VALIDATE_DATA = "@@xform-validate-data";

/**
 * 字段校验
 */
var XFORM_FIELD_VALIDATE = "@@xform-field-validate";

/**
 * 禁用字段的校验规则
 */
var XFORM_FIELD_VALIDATOR_DISABLE = "@@xform-field-validator-disable";

/**
 * 禁用字段校验的指定rule
 */
var XFORM_FIELD_VALIDATOR_RULE_DISABLE = "@@xform-field-validator-rule-disable";

/**
 * 清除表单的校验
 */
var XFORM_CLEAR_VALIDATE = "@@xform-clear-validate";

/**
 * 清除表单字段的校验
 */
var XFORM_CLEAR_FIELD_VALIDATE = "@@xform-clear-field-validate";

/**
 * 表单控件可见数据常量
 */
var XFORM_DISPLAY_DATA = "@@xform-display-data";

/**
 * 表单变更
 */
var XFORM_FIELD_CHANGE = "@@xform-field-change";

var XFORM_VALIDATE = "@@xfrom-validate";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

/**
 * 创建校验信息更新的payloads
 * @param {Obeject} payload 触发的action
 * @param {Object} state State
 * @param {Object} resultMap {
 *  name:{
 *    help:'校验失败',
 *    status:'error',
 *    hasFeedback:true
 *  }
 * }结果
 */
var createValidatePayload = function createValidatePayload(payload, getState, resultMap) {
  var formName = payload.formName;

  var newPayload = {};
  var state = getState();

  var stateValidateData = _extends({}, state[XFORM_VALIDATE_DATA]);
  var formValidateData = _extends({}, stateValidateData[formName], resultMap);

  stateValidateData[formName] = formValidateData;
  newPayload[XFORM_VALIDATE_DATA] = stateValidateData;
  return newPayload;
};

/**
 * 根据表单名称和字段名称获取校验信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFieldValidateData = function getFieldValidateData(state, formName, field) {
  var formValidateData = getFormValidateData(state, formName);
  return formValidateData[field];
};

/**
 * 根据表单名称获取校验信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFormValidateData = function getFormValidateData(state, formName) {
  var formValidateData = state[XFORM_VALIDATE_DATA] || {};
  return formValidateData[formName] || {};
};

/**
 * 创建值更新的payload
 * @param {Object} payload 触发的action
 * @param {Object} state State
 */
var createFieldValuePayload = function createFieldValuePayload(payload, getState) {
  var field = payload.field,
      value = payload.value,
      formName = payload.formName;

  var newPayload = {};
  var state = getState();

  if (formName) {
    newPayload[formName] = _extends({}, state[formName]);

    newPayload[formName][field] = value;
  } else {
    newPayload[field] = value;
  }
  return newPayload;
};

/**
 * 创建校验信息更新的payloads
 * @param {Obeject} payload 触发的action
 * @param {Object} state State
 * @param {Object} result {a:true,b:false}结果
 */
var createDisplayPayload = function createDisplayPayload(payload, getState, result) {
  var formName = payload.formName;

  var newPayload = {};
  var state = getState();

  var stateDisplayData = _extends({}, state[XFORM_DISPLAY_DATA]);
  var formDisplayData = _extends({}, stateDisplayData[formName], result);

  stateDisplayData[formName] = formDisplayData;
  newPayload[XFORM_DISPLAY_DATA] = stateDisplayData;
  return newPayload;
};

/**
 * 根据表单名称和字段名称获取展示信息
 * @param {Object} state State,store.getState()[namespace]获取的值
 * @param {String} formName 表单名称
 * @param {String} field 字段名称
 */
var getFieldDisplayData = function getFieldDisplayData(state, formName, field) {
  var formValidateData = state[XFORM_DISPLAY_DATA] || {};
  formValidateData = formValidateData[formName] || {};

  return formValidateData[field];
};

var _arguments = arguments;

var formInsMap = {};
var addForm = function addForm(ins) {
  var namespace = ins.context.getNamespace();
  var formName = ins.props.name;

  formInsMap[namespace + "/" + formName] = ins;
};
var removeForm = function removeForm(namespace, formName) {
  delete formInsMap[namespace + "/" + formName];
};
var getForm = function getForm(namespace, formName) {
  return formInsMap[namespace + "/" + formName];
};
var runFormMethod = function runFormMethod(namespace, formName, method) {
  var ins = getForm(namespace, formName);

  if (ins) {
    return ins[method] && ins[method]();
  }
};
var validate = function validate(namespace, formName, callback) {
  if (typeof formName == "function" && _arguments.length == 2) {
    callback = formName;
    formName = undefined;
  }
  var ins = getForm(namespace, formName);

  if (ins) {
    ins.validate(callback);
  }
};
var getFormData = function getFormData(namespace, formName) {
  return runFormMethod(namespace, formName, "getData");
};
var resetForm = function resetForm(namespace, formName) {
  return runFormMethod(namespace, formName, "reset");
};

var Form = function (_React$Component) {
  inherits(Form, _React$Component);

  function Form(props) {
    classCallCheck(this, Form);

    var _this = possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).call(this, props));

    _this.fieldList = [];
    _this.vFieldMap = {};
    return _this;
  }

  createClass(Form, [{
    key: "dispatch",
    value: function dispatch(action) {
      var dispatch = this.props.dispatch || this.context.dispatch;

      dispatch && dispatch(action);
    }
  }, {
    key: "getState",
    value: function getState() {
      var getState = this.context.getState;

      if (getState) {
        return getState();
      }

      return this.props;
    }
    /**
     * 根据字段名称，获取字段的展示数据
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldDisplayData",
    value: function getFieldDisplayData$$1(field) {
      var formName = this.props.name;

      var state = this.getState();
      return getFieldDisplayData(state, formName, field);
    }
    /**
     * 根据字段名称，获取字段的校验数据
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldVaidateData",
    value: function getFieldVaidateData(field) {
      var formName = this.props.name;

      var state = this.getState();
      return getFieldValidateData(state, formName, field);
    }
    /**
     * 字段值发生变化时触发
     * @param {String} field 字段名称，表单内不可重复
     * @param {String} value 值
     * @param {String} vField 字段类型，表单内可重复
     */

  }, {
    key: "onFieldChange",
    value: function onFieldChange(_ref) {
      var _this2 = this;

      var field = _ref.field,
          value = _ref.value,
          vField = _ref.vField;
      var formName = this.props.name;


      var payload = {
        field: field,
        vField: vField,
        value: value,
        formName: formName
      };
      // 受控组件：传递值
      this.dispatch({
        type: XFORM_FIELD_CHANGE,
        payload: payload
      });

      // 执行表单校验，连续操作只触发一次校验
      if (this.fieldChangeValidateSeed) {
        clearTimeout(this.fieldChangeValidateSeed);
      }
      this.fieldChangeValidateSeed = setTimeout(function () {
        _this2.validateField(payload);
      }, 400);
    }
  }, {
    key: "onFieldFocus",
    value: function onFieldFocus(_ref2) {
      var field = _ref2.field,
          vField = _ref2.vField;
    }
  }, {
    key: "onFieldBlur",
    value: function onFieldBlur() {}
  }, {
    key: "onFieldKeyUp",
    value: function onFieldKeyUp() {}
  }, {
    key: "onFieldDown",
    value: function onFieldDown() {}
    /**
     * 获取表单值
     */

  }, {
    key: "getData",
    value: function getData() {
      var formName = this.props.name;

      var state = this.getState();

      return state[formName] || {};
    }
  }, {
    key: "reset",
    value: function reset() {
      var namespace = this.context.getNamespace();
      var formName = this.props.name;

      var Model = KOS.getModel(namespace);

      var initialModel = Model.getInitial();
      this.setData(initialModel[formName]);
    }
  }, {
    key: "setData",
    value: function setData(formData) {
      var formName = this.props.name;


      this.dispatch({
        type: "setState",
        payload: defineProperty({}, formName, formData)
      });
    }
    /**
     * 根据字段名称，获取字段的值
     * @param {String} field 字段名称
     */

  }, {
    key: "getFieldValue",
    value: function getFieldValue(field) {
      var formData = this.getData();
      return formData[field];
    }
    /**
     * 执行全表单校验
     */

  }, {
    key: "validate",
    value: function validate(callback) {
      var formName = this.props.name;
      var vFieldMap = this.vFieldMap,
          fieldList = this.fieldList;

      this.dispatch({
        type: XFORM_VALIDATE,
        payload: {
          formName: formName,
          vFieldMap: vFieldMap,
          fieldList: fieldList,
          callback: callback
        }
      });
    }
    /**
     * 执行表单字段校验
     * @param {String|Object} field 校验的参数
     * @param {Function} callback 回调方法
     */

  }, {
    key: "validateField",
    value: function validateField(field, callback) {
      var payload = {
        value: this.getFieldValue(field),
        formName: this.props.name
      };
      if ((typeof field === "undefined" ? "undefined" : _typeof(field)) === "object") {
        Object.assign(payload, field);
      } else {
        Object.assign(payload, {
          field: field,
          vField: field
        });
      }

      this.dispatch({
        type: XFORM_FIELD_VALIDATE,
        payload: _extends({}, payload, {
          callback: callback
        })
      });
    }
    /**
     * 禁用字段校验
     * @param {String} field 字段名
     * @param {Boolean} disabled 是否禁用
     */

  }, {
    key: "disableFieldValidator",
    value: function disableFieldValidator(field, disabled) {
      var formName = this.props.name;

      this.dispatch({
        type: XFORM_FIELD_VALIDATOR_DISABLE,
        payload: {
          field: field,
          formName: formName,
          disabled: disabled
        }
      });
    }
    /**
     * 禁用字段校验的指定规则
     * @param {String} field 字段名
     * @param {String} rule 校验规则，可以是校验规则名，也可以是校验规则的index，从0开始
     * @param {Boolean} disabled 是否禁用
     */

  }, {
    key: "disableFieldValidatorRule",
    value: function disableFieldValidatorRule(field, rule, disabled) {
      var formName = this.props.name;

      this.dispatch({
        type: XFORM_FIELD_VALIDATOR_RULE_DISABLE,
        payload: {
          field: field,
          formName: formName,
          rule: rule,
          disabled: disabled
        }
      });
    }
  }, {
    key: "clearValidate",
    value: function clearValidate() {
      var formName = this.props.name;
      var fieldList = this.fieldList,
          vFieldMap = this.vFieldMap;

      this.dispatch({
        type: XFORM_CLEAR_VALIDATE,
        payload: { formName: formName, fieldList: fieldList, vFieldMap: vFieldMap }
      });
    }
  }, {
    key: "clearFieldValidate",
    value: function clearFieldValidate(field) {
      var formName = this.props.name;

      this.dispatch({
        type: XFORM_CLEAR_FIELD_VALIDATE,
        payload: { formName: formName, field: field }
      });
    }
  }, {
    key: "onSubmit",
    value: function onSubmit() {
      var _this3 = this;

      var onSubmit = this.props.onSubmit;


      this.validate(function (result) {
        var formData = _this3.getData();
        result && onSubmit && onSubmit(formData);
      });

      return false;
    }
  }, {
    key: "registerField",
    value: function registerField(field, vField) {
      if (field) {
        this.fieldList.push(field);

        if (vField) {
          this.vFieldMap[field] = vField;
        }
      }
    }
  }, {
    key: "revokeField",
    value: function revokeField(field) {
      var index = this.fieldList.indexOf(field);
      if (index > -1) {
        this.fieldList.splice(index, 1);
      }

      delete this.vFieldMap[field];
    }
  }, {
    key: "getChildContext",
    value: function getChildContext() {
      return {
        registerField: this.registerField.bind(this),
        revokeField: this.revokeField.bind(this),
        onFieldChange: this.onFieldChange.bind(this),
        getFieldValue: this.getFieldValue.bind(this),
        getFieldVaidateData: this.getFieldVaidateData.bind(this),
        getFieldDisplayData: this.getFieldDisplayData.bind(this)
      };
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      addForm(this);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var namespace = this.context.getNamespace();
      var formName = this.props.name;


      removeForm(namespace, formName);
    }
  }, {
    key: "render",
    value: function render() {
      var _this4 = this;

      var children = this.props.children;

      return React.createElement(
        "form",
        _extends({}, this.props, {
          onSubmit: function onSubmit() {
            return _this4.onSubmit();
          }
        }),
        children
      );
    }
  }]);
  return Form;
}(React.Component);

// 传递给field用的


Form.propTypes = {
  name: PropTypes.string.isRequired,
  onSubmit: PropTypes.func,
  dispatch: PropTypes.func
};
Form.childContextTypes = {
  registerField: PropTypes.func,
  revokeField: PropTypes.func,
  onFieldChange: PropTypes.func,
  getFieldValue: PropTypes.func,
  getFieldVaidateData: PropTypes.func,
  getFieldDisplayData: PropTypes.func
};

// 来自wrapper
Form.contextTypes = {
  dispatch: PropTypes.func,
  getState: PropTypes.func,
  getNamespace: PropTypes.func
};

Form.getForm = getForm;
Form.getFormData = getFormData;
Form.resetForm = resetForm;
Form.validate = validate;

KOS.wrapperProps({
  getForm: function getForm(formName) {
    var namespace = this.getNamespace();

    return Form.getForm(namespace, formName);
  }
});

var xField = (function (_ref) {
  var FieldWrapper = _ref.FieldWrapper,
      FieldProps = _ref.FieldProps;

  var Field = function (_React$PureComponent) {
    inherits(Field, _React$PureComponent);

    function Field(props) {
      classCallCheck(this, Field);

      var _this = possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this, props));

      _this.state = {};
      return _this;
    }

    createClass(Field, [{
      key: "componentDidMount",
      value: function componentDidMount() {
        var _props = this.props,
            vField = _props.vField,
            field = _props.field;

        this.context && this.context.registerField(field, vField);
      }
    }, {
      key: "componentWillUnmount",
      value: function componentWillUnmount() {
        var _props2 = this.props,
            vField = _props2.vField,
            field = _props2.field;

        this.context && this.context.revokeField(field, vField);
      }
    }, {
      key: "getFieldValue",
      value: function getFieldValue() {
        var field = this.props.field;


        return this.isOnComposition ? this.state.inputValue : this.context.getFieldValue(field);
      }
    }, {
      key: "getValidateData",
      value: function getValidateData() {
        var field = this.props.field;

        return this.context.getFieldVaidateData(field);
      }
    }, {
      key: "getDisplay",
      value: function getDisplay() {
        var field = this.props.field;

        return this.context.getFieldDisplayData(field) === false ? false : true;
      }
    }, {
      key: "onFieldChange",
      value: function onFieldChange(onChange) {
        return function (e) {
          var _props3 = this.props,
              field = _props3.field,
              getOnChangeValue = _props3.getOnChangeValue,
              vField = _props3.vField;

          var value = getOnChangeValue.apply(this, arguments);

          if (this.isOnComposition) {
            this.innerValue = value;
            this.setState({
              inputValue: value
            });
          } else {
            // 是input，记录input的光标的位置
            if (e.target instanceof HTMLInputElement) {
              this.inputSelection = {
                start: e.target.selectionStart,
                end: e.target.selectionEnd
              };
              this.inputTarget = e.target;
            }

            onChange && onChange.apply(this, arguments);
            this.context.onFieldChange({ field: field, value: value, vField: vField }, e);
          }
        }.bind(this);
      }
    }, {
      key: "onCompositionHandler",
      value: function onCompositionHandler(type, onChange, e) {
        switch (type) {
          case "start":
            this.isOnComposition = true;
            break;
          case "update":
            this.isOnComposition = true;
            break;
          case "end":
            this.isOnComposition = false;
            if (e.target instanceof HTMLInputElement && !this.isOnComposition) {
              this.onFieldChange(onChange)(e);
            }
            break;
        }
      }
    }, {
      key: "onFieldFocus",
      value: function onFieldFocus(onFocus) {
        var _this2 = this,
            _arguments = arguments;

        return function (e) {
          onFocus && onFocus.apply(_this2, _arguments);
        };
      }
    }, {
      key: "onFieldKeyDown",
      value: function onFieldKeyDown(onKeyDown) {}
    }, {
      key: "onFieldKeyUp",
      value: function onFieldKeyUp(onKeyUp) {}
    }, {
      key: "onFieldBlur",
      value: function onFieldBlur(onBlur) {}
    }, {
      key: "componentDidUpdate",
      value: function componentDidUpdate(prevProps) {
        // const { value } = prevProps;
        var value = this.getFieldValue();
        var inputSelection = this.inputSelection,
            inputTarget = this.inputTarget;

        if (inputSelection && inputTarget) {
          // 在 didUpdate 时根据情况恢复光标的位置
          // 如果光标的位置小于值的长度，那么可以判定属于中间编辑的情况
          // 此时恢复光标的位置
          if (inputSelection.start < value.length) {
            inputTarget.selectionStart = inputSelection.start;
            inputTarget.selectionEnd = inputSelection.end;
            this.inputSelection = null;
            this.inputTarget = null;
          }
        }
      }
    }, {
      key: "render",
      value: function render() {
        var _this3 = this;

        var isDisplay = this.getDisplay();
        if (!isDisplay) {
          return null;
        }

        var children = this.props.children;

        var validateData = this.getValidateData();

        var fieldProps = _extends({}, FieldProps, this.props, validateData);

        var onCompositionHandler = this.onCompositionHandler;

        var value = this.getFieldValue();

        return React.createElement(
          FieldWrapper,
          fieldProps,
          React.Children.map(children, function (child, index) {
            var _child$props = child.props,
                onChange = _child$props.onChange,
                onFocus = _child$props.onFocus,
                onBlur = _child$props.onBlur,
                onKeyDown = _child$props.onKeyDown,
                onKeyUp = _child$props.onKeyUp;

            // const onFieldChange = this.onFieldChange(onChange);

            var props = _extends({}, child.props, {
              value: value,
              onChange: _this3.onFieldChange(onChange),
              // onFocus: this.onFieldFocus(onFocus),
              // onBlur: this.onFieldBlur(onBlur),
              // onKeyDown: this.onFieldKeyDown(onKeyDown),
              // onKeyUp: this.onFieldKeyUp(onKeyUp),
              onCompositionStart: onCompositionHandler.bind(_this3, "start", onChange),
              onCompositionUpdate: onCompositionHandler.bind(_this3, "update", onChange),
              onCompositionEnd: onCompositionHandler.bind(_this3, "end", onChange)
            });
            return React.createElement(child.type, props);
          })
        );
      }
    }]);
    return Field;
  }(React.PureComponent);

  Field.defaultProps = {
    getOnChangeValue: function getOnChangeValue(e) {
      if (e && e.target) {
        return e.target.value;
      }
      return e;
    }
  };
  Field.propTypes = {
    field: PropTypes.string,
    vField: PropTypes.string,
    validator: PropTypes.any,
    getOnChangeValue: PropTypes.func
  };


  Field.contextTypes = {
    registerField: PropTypes.func,
    revokeField: PropTypes.func,
    onFieldChange: PropTypes.func,
    getFieldValue: PropTypes.func,
    getFieldVaidateData: PropTypes.func,
    getFieldDisplayData: PropTypes.func
  };

  return Field;
});

var formatStr = function formatStr(str, data) {
  data = [].concat(data);
  return str.replace(/\{(\d+)\}/g, function (m, i) {
    return data[i];
  });
};
var getValidateHelp = function getValidateHelp() {
  var help = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";
  var status = arguments[1];
  var data = arguments[2];

  var h = "";
  if ($.isString(help)) {
    h = status === "success" ? "" : help;
  } else {
    h = help && help[status];
  }

  return formatStr(h, data);
};

var _this = undefined;

/**
 * 支持的校验配置规则：
[{
  field:'name',
  rules:['required']
},{
  field:'name',
  rules:['maxLength:10']
},{
  field:'email',
  rules:['email']
},{
  field:'age',
  rules:[/d+/],
  help:'必须是数字'
},{
  field:'regpassword',
  data:[1234],
  fn(state,{field,value,formName},data){
    return true;
  }
}]
 */

var RuleParseUtil = {
  /**
   * 解析stringRule
   * @param {String} rule 规则，如：maxLength:2@最大长度不能超过{0};minLength min:1;rangeLength:1,2
   */
  stringParser: function stringParser(rule) {
    if (!rule) {
      return [];
    }
    rule = rule.split(";");
    var list = [];

    rule.forEach(function (item) {
      var _item$split = item.split("@"),
          _item$split2 = slicedToArray(_item$split, 2),
          express = _item$split2[0],
          help = _item$split2[1];

      var _express$split = express.split(":"),
          _express$split2 = slicedToArray(_express$split, 2),
          name = _express$split2[0],
          data = _express$split2[1];

      var v = _extends({}, getRule(name));

      // 转换成数组
      if (data) {
        v.data = data.split(",");
      }
      if (help) {
        v.help = help;
      }

      list.push(v);
    });

    return list;
  },
  functionParser: function functionParser(validator) {
    return {
      name: "fn",
      fn: validator
    };
  },
  regexpParser: function regexpParser(validator) {
    return _extends({}, getRule("regexp"), {
      data: validator
    });
  },
  objectParser: function objectParser(validator) {
    var name = validator.name,
        help = validator.help,
        fn = validator.fn;


    if (name) {
      return _extends({}, getRule(name), validator);
    }

    return _extends({
      name: "custome"
    }, validator);
  }
};

/**
 * 解析校验规则
 * @param {String|Function|RegExp|Object} rule 校验规则
 * @param {String|Object} custHelp 自定义文案
 */
var parseRule = function parseRule(rule, custHelp) {
  var v = null;
  if ($.isString(rule)) {
    v = RuleParseUtil.stringParser(rule);
  } else if ($.isFunction(rule)) {
    v = RuleParseUtil.functionParser(rule);
  } else if ($.isRegExp(rule)) {
    v = RuleParseUtil.regexpParser(rule);
  } else if ($.isObject(rule)) {
    v = RuleParseUtil.objectParser(rule);
  }

  if (v && custHelp !== undefined) {
    v.help = custHelp;
  }

  return v;
};

/**
 * 解析校验规则，可以一次解析多个
 * @param {Object} validatorItem
 * @example
 * {
 *  field:'',
 *  rules:['required',(value)=>{return value>10},/d+/g]
 * }
 */
var parseValidatorRules = function parseValidatorRules(fieldValidator) {
  var field = fieldValidator.field,
      help = fieldValidator.help;

  var list = [];
  var rules = [].concat(fieldValidator.rules);

  rules.forEach(function (ruleItem) {
    list = list.concat(parseRule(ruleItem, help));
  });

  return list;
};

var validatorsMap = {};

/**
 *
 * @param {String} name rule名称
 * @param {Function} fn 校验函数
 * @param {String|Object} help 帮助信息
 */
var addRule = function addRule(name, fn, help) {
  validatorsMap[name] = {
    name: name,
    fn: fn,
    help: help
  };
};

/**
 * 根据名称移除rule规则
 * @param {String} name rule名称
 */
var removeRule = function removeRule(name) {
  delete validatorsMap[name];
};

/**
 * 根据名称获取rule规则
 * @param {String} name rule名称
 */
var getRule = function getRule(name) {
  return validatorsMap[name];
};

/**
 *
 * @param {Object} rule 校验器
 * @param {Object} payload 参数
 * @param {Object} state 当前namespace下的Model
 */
var runRule = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, payload, rule) {
    var fn, data, disabled, _rule$help, help, result, validateStatus;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            fn = rule.fn, data = rule.data, disabled = rule.disabled;
            _rule$help = rule.help, help = _rule$help === undefined ? "校验错误！" : _rule$help;

            if (!(disabled || !fn)) {
              _context.next = 4;
              break;
            }

            return _context.abrupt("return", null);

          case 4:
            _context.next = 6;
            return fn(dispatch, getState, _extends({}, payload, {
              data: data
            }));

          case 6:
            result = _context.sent;

            if (!(result === undefined || result === null)) {
              _context.next = 9;
              break;
            }

            return _context.abrupt("return", null);

          case 9:
            validateStatus = "success";

            if (!$.isBoolean(result)) {
              _context.next = 14;
              break;
            }

            validateStatus = result ? "success" : "error";
            _context.next = 21;
            break;

          case 14:
            if (!$.isString(result)) {
              _context.next = 19;
              break;
            }

            // 返回的是错误消息
            help = result;
            if (result) {
              validateStatus = result ? "error" : "success";
            }
            _context.next = 21;
            break;

          case 19:
            if (!$.isObject(result)) {
              _context.next = 21;
              break;
            }

            return _context.abrupt("return", result);

          case 21:
            return _context.abrupt("return", {
              validateStatus: validateStatus,
              hasFeedback: true,
              help: getValidateHelp(help, validateStatus, data)
            });

          case 22:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, _this);
  }));

  return function runRule(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var ValidatorRule = /*#__PURE__*/Object.freeze({
  parseRule: parseRule,
  parseValidatorRules: parseValidatorRules,
  addRule: addRule,
  removeRule: removeRule,
  getRule: getRule,
  runRule: runRule
});

var FieldValidator = function () {
  function FieldValidator(field, rules) {
    classCallCheck(this, FieldValidator);

    this.field = field;
    this.disabled = false;
    this.initRules(rules);
  }
  /**
   * 初始化Rule
   * @param {Array<Object>} rules rule列表
   */


  createClass(FieldValidator, [{
    key: "initRules",
    value: function initRules(rules) {
      var list = [];
      rules.forEach(function (item) {
        list = list.concat(item);
      });
      this.rules = list;
    }
    /**
     * 禁用字段校验器
     * @param {Boolean} disable 是否禁用
     */

  }, {
    key: "setDisable",
    value: function setDisable() {
      var disabled = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

      this.disabled = disabled;
    }
    /**
     * 禁用校验规则
     * @param {String|Number} rule 校验规则名称或者序号，需要不推荐使用
     * @param {Boolean} disabled 是否禁用
     */

  }, {
    key: "setRuleDisable",
    value: function setRuleDisable(rule) {
      var disabled = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      var isNumber = typeof rule === "number";

      this.rules.forEach(function (ruleItem, index) {
        if (isNumber && index === rule) {
          ruleItem.disabled = disabled;
        } else if (ruleItem.name === rule) {
          ruleItem.disabled = disabled;
        }
      });
    }
    /**
     * 执行校验规则，返回执行状态
     * @param {Function} dispatch 校验状态更新器
     * @param {String} getState 获取当前State的方法
     * @param {Object} payload 参数，包括：field,vField,formName,data,value等
     * @returns validateResult 校验结果：{validateStatus,hasFeedback,help}
     */

  }, {
    key: "run",
    value: function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, payload) {
        var _payload$successFeedb, successFeedback, _payload$successHelp, successHelp, validateResult, rules, disabled, i, len, rule, result;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _payload$successFeedb = payload.successFeedback, successFeedback = _payload$successFeedb === undefined ? false : _payload$successFeedb, _payload$successHelp = payload.successHelp, successHelp = _payload$successHelp === undefined ? "" : _payload$successHelp;
                validateResult = {
                  hasFeedback: successFeedback,
                  validateStatus: "success",
                  help: successHelp
                };
                rules = this.rules, disabled = this.disabled;

                if (!disabled) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt("return", validateResult);

              case 5:
                i = 0, len = rules.length;

              case 6:
                if (!(i < len)) {
                  _context.next = 19;
                  break;
                }

                rule = rules[i];

                if (!(rule && rule.fn && !rule.disabled)) {
                  _context.next = 16;
                  break;
                }

                _context.next = 11;
                return runRule(dispatch, getState, payload, rule);

              case 11:
                result = _context.sent;

                if (!result) {
                  _context.next = 16;
                  break;
                }

                validateResult = result;

                if (!(result.validateStatus === "error")) {
                  _context.next = 16;
                  break;
                }

                return _context.abrupt("break", 19);

              case 16:
                i++;
                _context.next = 6;
                break;

              case 19:
                return _context.abrupt("return", validateResult);

              case 20:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function run(_x3, _x4, _x5) {
        return _ref.apply(this, arguments);
      }

      return run;
    }()
  }]);
  return FieldValidator;
}();

FieldValidator.factory = function (validators) {
  var vas = {};

  validators.forEach(function (fieldValidator) {
    var field = fieldValidator.field;

    var rules = parseValidatorRules(fieldValidator);

    vas[field] = vas[field] || [];
    vas[field] = vas[field].concat(rules);
  });

  // 转换成对象进行管理
  for (var field in vas) {
    var rules = vas[field];
    if (field && rules) {
      vas[field] = new FieldValidator(field, rules);
    }
  }

  return vas;
};

var FormValidator = function () {
  /**
   * 表单校验
   * @param {Array<Validator>} validators 校验规则配置
   * @param {String} formName 表单名
   */
  function FormValidator() {
    var validators = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var formName = arguments[1];
    classCallCheck(this, FormValidator);

    this.formName = formName;

    // 实例化字段的校验规则
    this.fieldValidatorMap = FieldValidator.factory(validators);
  }
  /**
   * 根据字段名，获取校验规则
   * @param {String} field 字段名
   */


  createClass(FormValidator, [{
    key: "getFieldValidator",
    value: function getFieldValidator(field) {
      return this.fieldValidatorMap[field];
    }
    /**
     * 禁用指定字段的校验规则
     * @param {String} field 字段名
     * @param {Boolean} disable 是否禁用
     */

  }, {
    key: "disableFieldValidator",
    value: function disableFieldValidator(field) {
      var disabled = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      var validator = this.getFieldValidator(field);
      if (validator) {
        validator.setDisable(disabled);
      }
    }
    /**
     * 禁用字段的指定校验规则
     * @param {String} field 字段名
     * @param {String|Number} rule 规则名称或者规则的序号
     * @param {Boolean} disable 是否禁用
     */

  }, {
    key: "disableFieldValidatorRule",
    value: function disableFieldValidatorRule(field, rule) {
      var disabled = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      var validator = this.getFieldValidator(field);
      if (validator) {
        validator.setRuleDisable(rule, disabled);
      }
    }
    /**
     * 执行所有的校验，并返回校验结果
     * @param {Object} payload 包含所有的 vFieldMap, fieldList
     * @param {Function} getState 获取完整State的方法
     */

  }, {
    key: "validate",
    value: function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, payload) {
        var formName, fieldValidatorMap, state, vFieldMap, fieldList, formData, fieldResult, formResult, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, field, vField, value, _payload, fieldValidateDispatch, result;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                formName = this.formName, fieldValidatorMap = this.fieldValidatorMap;
                state = getState();
                vFieldMap = payload.vFieldMap, fieldList = payload.fieldList;
                formData = (formName ? state[formName] : state) || {};
                fieldResult = {};
                formResult = true;

                // 此处循环的fieldList是表单的fieldList，即一个表单有多少个输入字段

                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 9;
                _iterator = fieldList[Symbol.iterator]();

              case 11:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context.next = 25;
                  break;
                }

                field = _step.value;
                vField = vFieldMap[field];
                value = formData[field];
                _payload = {
                  field: field,
                  vField: vField,
                  value: value,
                  formName: formName
                };
                // 返回一个只能执行更新字段校验信息的dispatch方法

                fieldValidateDispatch = dispatch(_payload);
                _context.next = 19;
                return this.validateField(fieldValidateDispatch, getState, _payload);

              case 19:
                result = _context.sent;


                fieldResult[field] = result;

                formResult = formResult && (result ? result.validateStatus !== "error" : true);

              case 22:
                _iteratorNormalCompletion = true;
                _context.next = 11;
                break;

              case 25:
                _context.next = 31;
                break;

              case 27:
                _context.prev = 27;
                _context.t0 = _context["catch"](9);
                _didIteratorError = true;
                _iteratorError = _context.t0;

              case 31:
                _context.prev = 31;
                _context.prev = 32;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 34:
                _context.prev = 34;

                if (!_didIteratorError) {
                  _context.next = 37;
                  break;
                }

                throw _iteratorError;

              case 37:
                return _context.finish(34);

              case 38:
                return _context.finish(31);

              case 39:
                return _context.abrupt("return", {
                  formResult: formResult,
                  fieldResult: fieldResult
                });

              case 40:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[9, 27, 31, 39], [32,, 34, 38]]);
      }));

      function validate(_x4, _x5, _x6) {
        return _ref.apply(this, arguments);
      }

      return validate;
    }()
    /**
     * 执行校验，
     * @param {Function} dispatch 可以更新校验状态的回调方法
     * @param {Function} getState 获取完整State的方法
     * @param {Object} payload 执行的上下文数据，包含field,vField字段,value
     */

  }, {
    key: "validateField",
    value: function () {
      var _ref2 = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dispatch, getState, payload) {
        var field, vField, fieldValidator;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                field = payload.field, vField = payload.vField;
                fieldValidator = vField ? this.getFieldValidator(vField) : this.getFieldValidator(field);

                if (!fieldValidator) {
                  _context2.next = 6;
                  break;
                }

                _context2.next = 5;
                return fieldValidator.run(dispatch, getState, payload);

              case 5:
                return _context2.abrupt("return", _context2.sent);

              case 6:
                return _context2.abrupt("return", null);

              case 7:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function validateField(_x7, _x8, _x9) {
        return _ref2.apply(this, arguments);
      }

      return validateField;
    }()
  }]);
  return FormValidator;
}();

/**
 * 工厂函数
 * @param validatorConfig，例如
 * 
 * [{
    formName: 'addForm',
    validators: [{
      field: 'page_type',
      rules: 'required'
    }, {
      field: 'page_desc',
      rules: ['required', (getState, { field, value }, data) => {
        return value && value.length <= 3;
      }],
      help: '请正确填写'
    }, {
      field: 'page_name',
      rules: ['required@好好填', {
        name: 'maxLength',
        data: 4,
        help: 'maxLength:{0}'
      }]
    }, {
      field: 'page_name',
      help: '异步校验失败',
      rules: {
        data: { a: 1 },
        fn: async (getState, { field, value }, data) => {
          const xdata = await fetch({
            url: '/app/list',
            data: {
            }
          });

          return true
        }
      }
    }]
  }]
 */

var Validator = function () {
  function Validator(validators) {
    classCallCheck(this, Validator);

    this.validators = validators;
    this.formValidator = {};
    this.init();
  }

  createClass(Validator, [{
    key: "init",
    value: function init() {
      var map = {};
      var validators = this.validators,
          formValidator = this.formValidator;

      // 第一层遍历formName，同一个formName可以写多个{formName,validators}的配置

      validators.forEach(function (item) {
        // 兼容只有一个表单，不写表单名的情况，使用时强烈推荐定义表单名
        var formName = item.formName,
            _item$validators = item.validators,
            validators = _item$validators === undefined ? item : _item$validators;

        map[formName] = map[formName] || [];
        map[formName] = map[formName].concat(validators);
      });

      // 根据formName的情况，进行校验规则的实例化
      for (var formName in map) {
        var _validators = map[formName];
        // 初始化
        formValidator[formName] = new FormValidator(_validators, formName);
      }
    }
    /**
     * 获取表单校验实例
     * @param {String} formName 表单名称
     */

  }, {
    key: "getFormValidator",
    value: function getFormValidator(formName) {
      return this.formValidator[formName];
    }
  }]);
  return Validator;
}();

Validator.create = function (validators) {
  return new Validator(validators);
};

var _this$1 = undefined;

var KOSUtil = KOS.Util;

/**
 * 获取Model的validatorIns
 * @param {Object} model model
 * @param {String} formName 表单名
 */
var getFormValidator = function getFormValidator(model, formName) {
  var validatorIns = model.validatorIns;


  if (!validatorIns) {
    validatorIns = Validator.create(model.getAttr("validators"));
    model.validatorIns = validatorIns;
  }

  return validatorIns.getFormValidator(formName);
};

var dispatchFieldValidate = function dispatchFieldValidate(dispatch, getState) {
  return function (payload, result) {
    var field = payload.field;

    if (result !== undefined && result !== null) {
      result.hasFeedback = result.hasFeedback === undefined ? true : result.hasFeedback;
      dispatch({
        type: "setState",
        payload: createValidatePayload(payload, getState, defineProperty({}, field, result))
      });
    }
  };
};

var updateFieldValidateStatus = function updateFieldValidateStatus(dispatch) {
  return function (payload) {
    return function (validateStatus) {
      var help = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";
      var hasFeedback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      if ((typeof validateStatus === "undefined" ? "undefined" : _typeof(validateStatus)) === "object") {
        dispatch(payload, _extends({
          hasFeedback: hasFeedback,
          help: help
        }, validateStatus));
      } else {
        dispatch(payload, {
          validateStatus: validateStatus,
          help: help,
          hasFeedback: hasFeedback
        });
      }
    };
  };
};

/**
 * 字段校验中间件
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体
 */
var fieldValidateMiddleware = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, action) {
    var _KOSUtil$getActionTyp, namespace, type, model, payload, formName, field, result, validatorIns, fieldValidateStatusDispatch, callback;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _KOSUtil$getActionTyp = KOSUtil.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;
            model = KOS.getModel(namespace);
            payload = action.payload;
            formName = payload.formName, field = payload.field;
            result = null;

            // 有表单校验相关配置

            validatorIns = getFormValidator(model, formName);
            fieldValidateStatusDispatch = dispatchFieldValidate(dispatch, getState);
            // console.log("run field validate", action);

            if (!validatorIns) {
              _context.next = 12;
              break;
            }

            _context.next = 10;
            return validatorIns.validateField(updateFieldValidateStatus(fieldValidateStatusDispatch)(payload), // 返回一个只能执行更新字段校验信息的dispatch方法
            getState, payload);

          case 10:
            result = _context.sent;


            fieldValidateStatusDispatch(payload, result);

          case 12:
            // 校验后进行回调
            callback = payload.callback;

            callback && callback(result);

            // return result;

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, _this$1);
  }));

  return function fieldValidateMiddleware(_x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * 表单校验中间件
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体，payload的
 */
var formValidateMiddleware = function () {
  var _ref2 = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dispatch, getState, action) {
    var _KOSUtil$getActionTyp2, namespace, model, payload, formName, validatorIns, formResult, fieldValidateStatusDispatch, result, callback;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _KOSUtil$getActionTyp2 = KOSUtil.getActionType(action.type), namespace = _KOSUtil$getActionTyp2.namespace;
            model = KOS.getModel(namespace);
            payload = action.payload;
            formName = payload.formName;
            validatorIns = getFormValidator(model, formName);
            formResult = true;

            if (!validatorIns) {
              _context2.next = 13;
              break;
            }

            fieldValidateStatusDispatch = dispatchFieldValidate(dispatch, getState);
            _context2.next = 10;
            return validatorIns.validate(updateFieldValidateStatus(fieldValidateStatusDispatch), getState, payload);

          case 10:
            result = _context2.sent;

            formResult = result.formResult;

            // 触发表单校验结果更新
            dispatch({
              type: "setState",
              payload: createValidatePayload(payload, getState, result.fieldResult)
            });

          case 13:

            // 校验后进行回调
            callback = payload.callback;

            callback && callback(formResult);

            return _context2.abrupt("return", formResult);

          case 16:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, _this$1);
  }));

  return function formValidateMiddleware(_x6, _x7, _x8) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * 禁止字段所有校验规则的中间件
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action action.payload格式如：{ formName, field, disable }
 */
var disableFieldValidatorMiddleware = function disableFieldValidatorMiddleware(dispatch, getState, action) {
  var _KOSUtil$getActionTyp3 = KOSUtil.getActionType(action.type),
      namespace = _KOSUtil$getActionTyp3.namespace,
      type = _KOSUtil$getActionTyp3.type;

  var _action$payload = action.payload,
      formName = _action$payload.formName,
      field = _action$payload.field,
      disabled = _action$payload.disabled;

  var model = KOS.getModel(namespace);

  var formValidatorIns = getFormValidator(model, formName);
  if (formValidatorIns && field) {
    formValidatorIns.disableFieldValidator(field, disabled);
  }
};

/**
 * 禁止字段指定校验规则的中间件
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action action.payload格式如：{ formName, field, rule, disable }
 */
var disableFieldValidatorRuleMiddleware = function disableFieldValidatorRuleMiddleware(dispatch, getState, action) {
  var _KOSUtil$getActionTyp4 = KOSUtil.getActionType(action.type),
      namespace = _KOSUtil$getActionTyp4.namespace,
      type = _KOSUtil$getActionTyp4.type;

  var _action$payload2 = action.payload,
      formName = _action$payload2.formName,
      field = _action$payload2.field,
      rule = _action$payload2.rule,
      disabled = _action$payload2.disabled;

  var model = KOS.getModel(namespace);

  var formValidatorIns = getFormValidator(model, formName);
  if (formValidatorIns && field && rule) {
    formValidatorIns.disableFieldValidatorRule(field, rule, disabled);
  }
};

var clearFormValidateMiddleware = function clearFormValidateMiddleware(dispatch, getState, action) {
  var _KOSUtil$getActionTyp5 = KOSUtil.getActionType(action.type),
      namespace = _KOSUtil$getActionTyp5.namespace,
      type = _KOSUtil$getActionTyp5.type;

  var payload = action.payload;
  var formName = payload.formName,
      fieldList = payload.fieldList,
      vFieldMap = payload.vFieldMap;


  var fieldValidateStatusDispatch = dispatchFieldValidate(dispatch, getState);
  var result = {
    validateStatus: "success",
    hasFeedback: false
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = fieldList[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var field = _step.value;

      fieldValidateStatusDispatch({ formName: formName, field: field }, result);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
};

var clearFieldValidateMiddleware = function clearFieldValidateMiddleware(dispatch, getState, action) {
  var payload = action.payload;
  var formName = payload.formName,
      field = payload.field;


  var fieldValidateStatusDispatch = dispatchFieldValidate(dispatch, getState);
  fieldValidateStatusDispatch(payload, {
    validateStatus: "success",
    hasFeedback: false
  });
};

var ValidatorMiddleware = /*#__PURE__*/Object.freeze({
  fieldValidateMiddleware: fieldValidateMiddleware,
  formValidateMiddleware: formValidateMiddleware,
  disableFieldValidatorMiddleware: disableFieldValidatorMiddleware,
  disableFieldValidatorRuleMiddleware: disableFieldValidatorRuleMiddleware,
  clearFormValidateMiddleware: clearFormValidateMiddleware,
  clearFieldValidateMiddleware: clearFieldValidateMiddleware
});

var addRule$1 = addRule;


addRule$1("required", function (dispatch, getState, _ref) {
  var value = _ref.value,
      field = _ref.field,
      data = _ref.data;

  return !!value;
}, "不能为空！");

addRule$1("maxLength", function (dispatch, getState, _ref2) {
  var value = _ref2.value,
      field = _ref2.field,
      maxLength = _ref2.data;

  if (value) {
    return value.length <= maxLength;
  }
  return true;
});

addRule$1("minLength", function (dispatch, getState, _ref3) {
  var value = _ref3.value,
      field = _ref3.field,
      minLength = _ref3.data;

  if (value) {
    return value.length >= minLength;
  }
  return true;
});
addRule$1("rangeLength", function (dispatch, getState, _ref4) {
  var value = _ref4.value,
      field = _ref4.field,
      rangeLength = _ref4.data;

  var _rangeLength = slicedToArray(rangeLength, 2),
      minLength = _rangeLength[0],
      maxLength = _rangeLength[1];

  if (value) {
    var len = value.length;
    return minLength <= len && len <= maxLength;
  }
  return true;
});

addRule$1("min", function (dispatch, getState, _ref5) {
  var value = _ref5.value,
      field = _ref5.field,
      min = _ref5.data;

  return value >= min;
});
addRule$1("max", function (dispatch, getState, _ref6) {
  var value = _ref6.value,
      field = _ref6.field,
      max = _ref6.data;

  return value <= max;
});
addRule$1("range", function (dispatch, getState, _ref7) {
  var value = _ref7.value,
      field = _ref7.field,
      range = _ref7.data;

  var _range = slicedToArray(range, 2),
      min = _range[0],
      max = _range[1];

  return !value || min <= value && value <= max;
});

addRule$1("regexp", function (dispatch, getState, _ref8) {
  var value = _ref8.value,
      field = _ref8.field,
      regExp = _ref8.data;

  return !value || regExp.test(value);
});

addRule$1("chinese", function (dispatch, getState, _ref9) {
  var value = _ref9.value,
      field = _ref9.field;

  return (/^[\u4e00-\u9fa5]*$/.test(value)
  );
});

addRule$1("email", function (dispatch, getState, _ref10) {
  var value = _ref10.value,
      field = _ref10.field,
      data = _ref10.data;

  return !value || /[a-z0-9-]{1,30}@[a-z0-9-]{1,65}.[a-z]{3}/.test(value);
}, "请输入正确的email地址");

var FormValidator$1 = _extends({
  /**
   * 校验器类
   */
  Validator: Validator
}, ValidatorMiddleware, ValidatorRule);

var _this$2 = undefined;

var KOSUtil$1 = KOS.Util;
/**
 * 
 * @param {Function} dispatch dispatch方法，包装了namesapce的
 * @param {Function} getState 获取当前namespace下的getState方法
 * @param {Object} action 包含type、payload的对象实体
 */
var fieldDisplayMiddleware = function () {
  var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(dispatch, getState, action) {
    var _KOSUtil$getActionTyp, namespace, type, model, payload, formName, field, formController, fieldDisplay, result;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _KOSUtil$getActionTyp = KOSUtil$1.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;
            model = KOS.getModel(namespace);
            payload = action.payload;
            formName = payload.formName, field = payload.field;
            formController = model.getAttr('formFieldDisplay') || {};

            if (formName) {
              formController = formController[formName] || {};
            }

            fieldDisplay = formController[field];

            if (!fieldDisplay) {
              _context.next = 13;
              break;
            }

            _context.next = 10;
            return fieldDisplay(getState, payload);

          case 10:
            result = _context.sent;


            // 触发表单校验结果更新
            result && dispatch({
              type: 'setState',
              payload: createDisplayPayload(payload, getState, result)
            });

            return _context.abrupt('return', result);

          case 13:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this$2);
  }));

  return function fieldDisplayMiddleware(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var _this$3 = undefined;
var fieldValidateMiddleware$1 = FormValidator$1.fieldValidateMiddleware,
    formValidateMiddleware$1 = FormValidator$1.formValidateMiddleware,
    disableFieldValidatorMiddleware$1 = FormValidator$1.disableFieldValidatorMiddleware,
    disableFieldValidatorRuleMiddleware$1 = FormValidator$1.disableFieldValidatorRuleMiddleware,
    clearFieldValidateMiddleware$1 = FormValidator$1.clearFieldValidateMiddleware,
    clearFormValidateMiddleware$1 = FormValidator$1.clearFormValidateMiddleware;

var KOSUtil$2 = KOS.Util;
// let fieldValidateSeed = null;
var FormMiddleware = function FormMiddleware(store) {
  return function (next) {
    return function () {
      var _ref = asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(action) {
        var _KOSUtil$getActionTyp, namespace, type, getState, dispatch;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _KOSUtil$getActionTyp = KOSUtil$2.getActionType(action.type), namespace = _KOSUtil$getActionTyp.namespace, type = _KOSUtil$getActionTyp.type;

                getState = function getState() {
                  return store.getState()[namespace];
                };

                dispatch = function dispatch(action) {
                  var type = action.type;

                  store.dispatch(_extends({}, action, {
                    type: namespace + "/" + type
                  }));
                };

                _context.next = 5;
                return next(action);

              case 5:
                _context.t0 = type;
                _context.next = _context.t0 === XFORM_FIELD_CHANGE ? 8 : _context.t0 === XFORM_FIELD_VALIDATE ? 10 : _context.t0 === XFORM_VALIDATE ? 12 : _context.t0 === XFORM_FIELD_VALIDATOR_DISABLE ? 15 : _context.t0 === XFORM_FIELD_VALIDATOR_RULE_DISABLE ? 17 : _context.t0 === XFORM_CLEAR_VALIDATE ? 19 : _context.t0 === XFORM_CLEAR_FIELD_VALIDATE ? 21 : 23;
                break;

              case 8:
                // 字段值变更
                // 更新值
                dispatch({
                  type: "setState",
                  payload: createFieldValuePayload(action.payload, getState)
                });
                return _context.abrupt("break", 23);

              case 10:
                // 字段校验
                fieldValidateMiddleware$1(dispatch, getState, action);
                return _context.abrupt("break", 23);

              case 12:
                _context.next = 14;
                return formValidateMiddleware$1(dispatch, getState, action);

              case 14:
                return _context.abrupt("break", 23);

              case 15:
                // 禁用字段所有的校验规则
                disableFieldValidatorMiddleware$1(dispatch, getState, action);
                return _context.abrupt("break", 23);

              case 17:
                // 禁用字段指定的校验规则
                disableFieldValidatorRuleMiddleware$1(dispatch, getState, action);
                return _context.abrupt("break", 23);

              case 19:
                // 清除表单的校验状态
                clearFormValidateMiddleware$1(dispatch, getState, action);
                return _context.abrupt("break", 23);

              case 21:
                // 清除表单指定字段的校验状态
                clearFieldValidateMiddleware$1(dispatch, getState, action);
                return _context.abrupt("break", 23);

              case 23:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, _this$3);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }();
  };
};

var Form$1 = Form;
var formMiddleware = FormMiddleware;
var FieldHOC = xField;
var addRule$2 = addRule;
var removeRule$1 = removeRule;

exports.Form = Form$1;
exports.formMiddleware = formMiddleware;
exports.FieldHOC = FieldHOC;
exports.addRule = addRule$2;
exports.removeRule = removeRule$1;
